#include <SPI.h>

const byte REGISTER_READ = B01000001; 
const byte REGISTER_WRITE = B01000000; 


/* INDIRIZZI DEI REGISTRI DELL'IO EXPANDER MCP23S17 */


#define		IODIRA		0x00		// REGISTRO I/O DIRECTION
#define		IODIRB		0x01		// 1=input

#define		IPOLA		  0x02		// POLARITA` I/O
#define		IPOLB		  0x03		// 1= Inverted

#define		GPIOA	  	0x12		// INDIRIZZO PORTA GP I/O
#define		GPIOB	  	0x13		// 

#define		GPINTENA	0x04		// ABILITA IOC (interrupt on change) 
#define		GPINTENB	0x05		// 1 = enabled 

#define		DEFVALA		0x06		// MCP23017 IOC Default value
#define		DEFVALB		0x07		// MCP23017 if INTCONA set then INT. if diff. 

#define		INTCONA		0x08		// INTERRUPT CONTROL REGISTER
#define		INTCONB		0x09		// 1 = INT IF INPUT DIFFER FROM DEFVAL  0=INTERRUPT ON PIN CHANGE

#define		IOCON		  0x0A		// REGISTRO DI CONFIGURAZIONE
//#define		IOCONB	  0x0B		// REGISTRO DI CONFIGURAZIONE  - NON IN USO CON IOCON.BANK=0

#define		GPPUA		  0x0C		// INTERNAL PULLUPS
#define		GPPUB	  	0x0D		// 1= PULLUP ABILITATO 

#define		INTFA		  0x0E            // INTERRUPT FLAG REGISTER
#define		INTFB		  0x0F            // 1 = PIN CAUSED INTERRUPT

#define		INTCAPA   0x10            // INTERRUPT CAPTURE VALUE
#define		INTCAPB		0x11            // GPIO PORT STATUS CAPTURED ON INTERRUPT

#define		GPIOA     0x12            // INTERRUPT CAPTURE VALUE
#define		GPIOB	  	0x13            // GPIO PORT STATUS CAPTURED ON INTERRUPT

#define		OLATA		  0x14            // ACCESS OUTPUT LATCHES    
#define		OLATB		  0x15            //

//pin per SPI e I/O expander
const byte pinCS  = 10;
const byte pinSCK = 13;
const byte pinSI  = 11;
const byte pinSO  = 12;
const byte pinRST = 8;

void setup() {

  pinMode(pinCS, OUTPUT);
  pinMode(pinSCK,OUTPUT);
  pinMode(pinSI, OUTPUT);
  pinMode(pinSO,  INPUT);
  pinMode(pinRST, OUTPUT);
  
  digitalWrite(pinCS, HIGH);
  // Inizializziamo l'IO expander
  digitalWrite(pinRST, LOW);
  delay(1000);
  digitalWrite(pinRST, HIGH);
  delay(100);
  SPI.begin();
  SPI.setBitOrder(MSBFIRST);
  SPI.setClockDivider(SPI_CLOCK_DIV2);
  SPI.setDataMode(SPI_MODE0);
  delay(1000);

  //configuriamo
  digitalWrite(pinCS, LOW);
  SPI.transfer(REGISTER_WRITE);
  SPI.transfer(IOCON);
  SPI.transfer(0x00); // all defaults, hwaddr disabled,seqop
  digitalWrite(pinCS, HIGH);
  
  // l'io expander per i led
  //output per i led
  digitalWrite(pinCS, LOW);
  SPI.transfer(REGISTER_WRITE);
  SPI.transfer(IODIRA);
  SPI.transfer(0x00); // IODIRA
  SPI.transfer(0x00); // IODIRB
  digitalWrite(pinCS, HIGH);
  
}


void loop(){
  
   for ( byte light = 1; light > 0; light <<= 1 ) 
   {
      digitalWrite(pinCS, LOW);
      SPI.transfer(REGISTER_WRITE);
      SPI.transfer(GPIOA);
      SPI.transfer(light); // GPIOA
      SPI.transfer(light); // GPIOB
      digitalWrite(pinCS, HIGH);
      delay(1000);
   }

}
